@@ -51,8 +51,8 @@ import org.apache.tomcat.util.http.CookieProcessor;
 /**
  * A <b>Context</b> is a Container that represents a servlet context, and therefore an individual web application, in
  * the Catalina servlet engine. It is therefore useful in almost every deployment of Catalina (even if a Connector
- * attached to a web server (such as Apache) uses the web server's facilities to identify the appropriate Wrapper to
- * handle this request. It also provides a convenient mechanism to use Interceptors that see every request processed by
+ * attached to a web server such as Apache uses the web server's facilities to identify the appropriate Wrapper to
+ * handle this request). It also provides a convenient mechanism to use Interceptors that see every request processed by
  * this particular web application.
  * <p>
  * The parent Container attached to a Context is generally a Host, but may be some other implementation, or may be
@@ -61,8 +61,6 @@ import org.apache.tomcat.util.http.CookieProcessor;
  * The child containers attached to a Context are generally implementations of Wrapper (representing individual servlet
  * definitions).
  * <p>
- *
- * @author Craig R. McClanahan
  */
 public interface Context extends Container, ContextBind {
 
@@ -131,7 +129,7 @@ public interface Context extends Container, ContextBind {
      *
      * @param listeners The set of instantiated listener objects.
      */
-    void setApplicationEventListeners(Object listeners[]);
+    void setApplicationEventListeners(Object[] listeners);
 
 
     /**
@@ -149,7 +147,7 @@ public interface Context extends Container, ContextBind {
      *
      * @param listeners The set of instantiated listener objects.
      */
-    void setApplicationLifecycleListeners(Object listeners[]);
+    void setApplicationLifecycleListeners(Object[] listeners);
 
 
     /**
@@ -317,7 +315,7 @@ public interface Context extends Container, ContextBind {
      * Configures if a / is added to the end of the session cookie path to ensure browsers, particularly IE, don't send
      * a session cookie for context /foo with requests intended for context /foobar.
      *
-     * @param sessionCookiePathUsesTrailingSlash <code>true</code> if the slash is should be added, otherwise
+     * @param sessionCookiePathUsesTrailingSlash <code>true</code> if the slash should be added, otherwise
      *                                               <code>false</code>
      */
     void setSessionCookiePathUsesTrailingSlash(boolean sessionCookiePathUsesTrailingSlash);
@@ -952,19 +950,19 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * @return the set of application listener class names configured for this application.
+     * @return the array of application listener class names configured for this application.
      */
     String[] findApplicationListeners();
 
 
     /**
-     * @return the set of application parameters for this application.
+     * @return the array of application parameters for this application.
      */
     ApplicationParameter[] findApplicationParameters();
 
 
     /**
-     * @return the set of security constraints for this web application. If there are none, a zero-length array is
+     * @return the array of security constraints for this web application. If there are none, a zero-length array is
      *             returned.
      */
     SecurityConstraint[] findConstraints();
@@ -991,7 +989,7 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * @return the set of defined error pages for all specified error codes and exception types.
+     * @return the array of defined error pages for all specified error codes and exception types.
      */
     ErrorPage[] findErrorPages();
 
@@ -1005,13 +1003,13 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * @return the set of defined filters for this Context.
+     * @return the array of defined filters for this Context.
      */
     FilterDef[] findFilterDefs();
 
 
     /**
-     * @return the set of filter mappings for this Context.
+     * @return the array of filter mappings for this Context.
      */
     FilterMap[] findFilterMaps();
 
@@ -1104,7 +1102,8 @@ public interface Context extends Container, ContextBind {
 
 
     /**
-     * @return the set of watched resources for this Context. If none are defined, a zero length array will be returned.
+     * @return the array of watched resources for this Context. If none are defined, a zero length array will be
+     *             returned.
      */
     String[] findWatchedResources();
 
@@ -1113,25 +1112,26 @@ public interface Context extends Container, ContextBind {
      * @return <code>true</code> if the specified welcome file is defined for this Context; otherwise return
      *             <code>false</code>.
      *
-     * @param name Welcome file to verify
+     * @param name the welcome file to verify
      */
     boolean findWelcomeFile(String name);
 
 
     /**
-     * @return the set of welcome files defined for this Context. If none are defined, a zero-length array is returned.
+     * @return the array of welcome files defined for this Context. If none are defined, a zero-length array is
+     *             returned.
      */
     String[] findWelcomeFiles();
 
 
     /**
-     * @return the set of LifecycleListener classes that will be added to newly created Wrappers automatically.
+     * @return the array of LifecycleListener classes that will be added to newly created Wrappers automatically.
      */
     String[] findWrapperLifecycles();
 
 
     /**
-     * @return the set of ContainerListener classes that will be added to newly created Wrappers automatically.
+     * @return the array of ContainerListener classes that will be added to newly created Wrappers automatically.
      */
     String[] findWrapperListeners();
 
