@@ -48,11 +48,9 @@ import jakarta.servlet.annotation.MultipartConfig;
 import org.apache.catalina.Container;
 import org.apache.catalina.ContainerServlet;
 import org.apache.catalina.Context;
-import org.apache.catalina.Globals;
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Wrapper;
-import org.apache.catalina.security.SecurityUtil;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.InstanceManager;
@@ -65,9 +63,6 @@ import org.apache.tomcat.util.modeler.Util;
 /**
  * Standard implementation of the <b>Wrapper</b> interface that represents an individual servlet definition. No child
  * Containers are allowed, and the parent Container must be a Context.
- *
- * @author Craig R. McClanahan
- * @author Remy Maucherat
  */
 public class StandardWrapper extends ContainerBase implements ServletConfig, Wrapper, NotificationEmitter {
 
@@ -222,11 +217,6 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
 
     private boolean overridable = false;
 
-    /**
-     * Static class array used when the SecurityManager is turned on and <code>Servlet.init</code> is invoked.
-     */
-    protected static Class<?>[] classType = new Class[] { ServletConfig.class };
-
     private final ReentrantReadWriteLock parametersLock = new ReentrantReadWriteLock();
 
     private final ReentrantReadWriteLock mappingsLock = new ReentrantReadWriteLock();
@@ -665,8 +655,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
         }
 
         // If not specified on the Wrapper, check the Context
-        if (getParent() instanceof Context) {
-            Context context = (Context) getParent();
+        if (getParent() instanceof Context context) {
             if (reference != null) {
                 reference = context.findRoleMapping(reference);
             } else {
@@ -827,22 +816,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
 
         // Call the initialization method of this servlet
         try {
-            if (Globals.IS_SECURITY_ENABLED) {
-                boolean success = false;
-                try {
-                    Object[] args = new Object[] { facade };
-                    SecurityUtil.doAsPrivilege("init", servlet, classType, args);
-                    success = true;
-                } finally {
-                    if (!success) {
-                        // destroy() will not be called, thus clear the reference now
-                        SecurityUtil.remove(servlet);
-                    }
-                }
-            } else {
-                servlet.init(facade);
-            }
-
+            servlet.init(facade);
             instanceInitialized = true;
         } catch (UnavailableException f) {
             unavailable(f);
@@ -956,16 +930,7 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
 
             // Call the servlet destroy() method
             try {
-                if (Globals.IS_SECURITY_ENABLED) {
-                    try {
-                        SecurityUtil.doAsPrivilege("destroy", instance);
-                    } finally {
-                        SecurityUtil.remove(instance);
-                    }
-                } else {
-                    instance.destroy();
-                }
-
+                instance.destroy();
             } catch (Throwable t) {
                 Throwable throwable = ExceptionUtils.unwrapInvocationTargetException(t);
                 ExceptionUtils.handleThrowable(throwable);
@@ -1065,12 +1030,8 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
      * Returns the number of requests processed by the wrapper.
      *
      * @return the number of requests processed by the wrapper.
-     *
-     * @deprecated The return type will change to long in Tomcat 11 onwards. Callers of this method should switch to
-     *                 storing the result of calls to this method in a long value rather than an int.
      */
-    @Deprecated
-    public int getRequestCount() {
+    public long getRequestCount() {
         return swValve.getRequestCount();
     }
 
@@ -1078,12 +1039,8 @@ public class StandardWrapper extends ContainerBase implements ServletConfig, Wra
      * Returns the number of requests processed by the wrapper that resulted in an error.
      *
      * @return the number of requests processed by the wrapper that resulted in an error.
-     *
-     * @deprecated The return type will change to long in Tomcat 11 onwards. Callers of this method should switch to
-     *                 storing the result of calls to this method in a long value rather than an int.
      */
-    @Deprecated
-    public int getErrorCount() {
+    public long getErrorCount() {
         return swValve.getErrorCount();
     }
 
