@@ -20,18 +20,13 @@ import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStreamReader;
 import java.io.PrintWriter;
+import java.util.StringTokenizer;
 
 import org.apache.catalina.util.IOTools;
 import org.apache.tomcat.util.res.StringManager;
 
 /**
- * Implements the Server-side #exec command
- *
- * @author Bip Thelin
- * @author Amy Roh
- * @author Paul Speed
- * @author Dan Sandberg
- * @author David Becker
+ * Implements the Server-side #exec command.
  */
 public class SSIExec implements SSICommand {
     private static final StringManager sm = StringManager.getManager(SSIExec.class);
@@ -54,7 +49,12 @@ public class SSIExec implements SSICommand {
             boolean foundProgram = false;
             try {
                 Runtime rt = Runtime.getRuntime();
-                Process proc = rt.exec(substitutedValue);
+                StringTokenizer st = new StringTokenizer(substitutedValue);
+                String[] cmdArray = new String[st.countTokens()];
+                for (int i = 0; i < cmdArray.length; i++) {
+                    cmdArray[i] = st.nextToken();
+                }
+                Process proc = rt.exec(cmdArray);
                 foundProgram = true;
                 char[] buf = new char[BUFFER_SIZE];
                 try (BufferedReader stdOutReader = new BufferedReader(new InputStreamReader(proc.getInputStream()));
@@ -78,4 +78,4 @@ public class SSIExec implements SSICommand {
         }
         return lastModified;
     }
-}
+}
\ No newline at end of file
