@@ -24,10 +24,7 @@ import org.apache.tomcat.util.ExceptionUtils;
 
 
 /**
- * Simple utility module to make it easy to plug in the server identifier
- * when integrating Tomcat.
- *
- * @author Craig R. McClanahan
+ * Simple utility module to make it easy to plug in the server identifier when integrating Tomcat.
  */
 public class ServerInfo {
 
@@ -63,8 +60,7 @@ public class ServerInfo {
         String number = null;
 
         Properties props = new Properties();
-        try (InputStream is = ServerInfo.class.getResourceAsStream
-                ("/org/apache/catalina/util/ServerInfo.properties")) {
+        try (InputStream is = ServerInfo.class.getResourceAsStream("/org/apache/catalina/util/ServerInfo.properties")) {
             props.load(is);
             info = props.getProperty("server.info");
             built = props.getProperty("server.built");
@@ -74,7 +70,7 @@ public class ServerInfo {
             ExceptionUtils.handleThrowable(t);
         }
         if (info == null || info.equals("Apache Tomcat/@VERSION@")) {
-            info = "Apache Tomcat/9.0.x-dev";
+            info = "Apache Tomcat/11.0.x-dev";
         }
         if (built == null || built.equals("@VERSION_BUILT@")) {
             built = "unknown";
@@ -83,7 +79,7 @@ public class ServerInfo {
             builtIso = "unknown";
         }
         if (number == null || number.equals("@VERSION_NUMBER@")) {
-            number = "9.0.x";
+            number = "11.0.x";
         }
 
         serverInfo = info;
@@ -124,20 +120,15 @@ public class ServerInfo {
         return serverNumber;
     }
 
-    public static void main(String args[]) {
+    public static void main(String[] args) {
         System.out.println("Server version: " + getServerInfo());
         System.out.println("Server built:   " + getServerBuilt());
         System.out.println("Server number:  " + getServerNumber());
-        System.out.println("OS Name:        " +
-                           System.getProperty("os.name"));
-        System.out.println("OS Version:     " +
-                           System.getProperty("os.version"));
-        System.out.println("Architecture:   " +
-                           System.getProperty("os.arch"));
-        System.out.println("JVM Version:    " +
-                           System.getProperty("java.runtime.version"));
-        System.out.println("JVM Vendor:     " +
-                           System.getProperty("java.vm.vendor"));
+        System.out.println("OS Name:        " + System.getProperty("os.name"));
+        System.out.println("OS Version:     " + System.getProperty("os.version"));
+        System.out.println("Architecture:   " + System.getProperty("os.arch"));
+        System.out.println("JVM Version:    " + System.getProperty("java.runtime.version"));
+        System.out.println("JVM Vendor:     " + System.getProperty("java.vm.vendor"));
     }
 
 }
