@@ -17,6 +17,7 @@
 package org.apache.catalina.valves;
 
 import java.io.IOException;
+import java.io.Serial;
 import java.io.Serializable;
 import java.util.Enumeration;
 import java.util.Map;
@@ -37,18 +38,15 @@ import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 
 /**
- * Web crawlers can trigger the creation of many thousands of sessions as they
- * crawl a site which may result in significant memory consumption. This Valve
- * ensures that crawlers are associated with a single session - just like normal
- * users - regardless of whether or not they provide a session token with their
- * requests.
+ * Web crawlers can trigger the creation of many thousands of sessions as they crawl a site which may result in
+ * significant memory consumption. This Valve ensures that crawlers are associated with a single session - just like
+ * normal users - regardless of whether or not they provide a session token with their requests.
  */
 public class CrawlerSessionManagerValve extends ValveBase {
 
     private static final Log log = LogFactory.getLog(CrawlerSessionManagerValve.class);
 
-    private final Map<String, String> clientIdSessionId = new ConcurrentHashMap<>();
-    private final Map<String, String> sessionIdClientId = new ConcurrentHashMap<>();
+    private final Map<String,String> clientIdSessionId = new ConcurrentHashMap<>();
 
     private String crawlerUserAgents = ".*[bB]ot.*|.*Yahoo! Slurp.*|.*Feedfetcher-Google.*";
     private Pattern uaPattern = null;
@@ -72,15 +70,14 @@ public class CrawlerSessionManagerValve extends ValveBase {
 
 
     /**
-     * Specify the regular expression (using {@link Pattern}) that will be used
-     * to identify crawlers based in the User-Agent header provided. The default
-     * is ".*GoogleBot.*|.*bingbot.*|.*Yahoo! Slurp.*"
+     * Specify the regular expression (using {@link Pattern}) that will be used to identify crawlers based in the
+     * User-Agent header provided. The default is ".*GoogleBot.*|.*bingbot.*|.*Yahoo! Slurp.*"
      *
      * @param crawlerUserAgents The regular expression using {@link Pattern}
      */
     public void setCrawlerUserAgents(String crawlerUserAgents) {
         this.crawlerUserAgents = crawlerUserAgents;
-        if (crawlerUserAgents == null || crawlerUserAgents.length() == 0) {
+        if (crawlerUserAgents == null || crawlerUserAgents.isEmpty()) {
             uaPattern = null;
         } else {
             uaPattern = Pattern.compile(crawlerUserAgents);
@@ -89,7 +86,8 @@ public class CrawlerSessionManagerValve extends ValveBase {
 
     /**
      * @see #setCrawlerUserAgents(String)
-     * @return  The current regular expression being used to match user agents.
+     *
+     * @return The current regular expression being used to match user agents.
      */
     public String getCrawlerUserAgents() {
         return crawlerUserAgents;
@@ -97,15 +95,14 @@ public class CrawlerSessionManagerValve extends ValveBase {
 
 
     /**
-     * Specify the regular expression (using {@link Pattern}) that will be used
-     * to identify crawlers based on their IP address. The default is no crawler
-     * IPs.
+     * Specify the regular expression (using {@link Pattern}) that will be used to identify crawlers based on their IP
+     * address. The default is no crawler IPs.
      *
      * @param crawlerIps The regular expression using {@link Pattern}
      */
     public void setCrawlerIps(String crawlerIps) {
         this.crawlerIps = crawlerIps;
-        if (crawlerIps == null || crawlerIps.length() == 0) {
+        if (crawlerIps == null || crawlerIps.isEmpty()) {
             ipPattern = null;
         } else {
             ipPattern = Pattern.compile(crawlerIps);
@@ -114,6 +111,7 @@ public class CrawlerSessionManagerValve extends ValveBase {
 
     /**
      * @see #setCrawlerIps(String)
+     *
      * @return The current regular expression being used to match IP addresses.
      */
     public String getCrawlerIps() {
@@ -122,10 +120,10 @@ public class CrawlerSessionManagerValve extends ValveBase {
 
 
     /**
-     * Specify the session timeout (in seconds) for a crawler's session. This is
-     * typically lower than that for a user session. The default is 60 seconds.
+     * Specify the session timeout (in seconds) for a crawler's session. This is typically lower than that for a user
+     * session. The default is 60 seconds.
      *
-     * @param sessionInactiveInterval   The new timeout for crawler sessions
+     * @param sessionInactiveInterval The new timeout for crawler sessions
      */
     public void setSessionInactiveInterval(int sessionInactiveInterval) {
         this.sessionInactiveInterval = sessionInactiveInterval;
@@ -133,14 +131,15 @@ public class CrawlerSessionManagerValve extends ValveBase {
 
     /**
      * @see #setSessionInactiveInterval(int)
-     * @return  The current timeout in seconds
+     *
+     * @return The current timeout in seconds
      */
     public int getSessionInactiveInterval() {
         return sessionInactiveInterval;
     }
 
 
-    public Map<String, String> getClientIpSessionId() {
+    public Map<String,String> getClientIpSessionId() {
         return clientIdSessionId;
     }
 
@@ -176,14 +175,21 @@ public class CrawlerSessionManagerValve extends ValveBase {
     @Override
     public void invoke(Request request, Response response) throws IOException, ServletException {
 
+        Host host = request.getHost();
+        if (host == null) {
+            // Request will have no session
+            getNext().invoke(request, response);
+            return;
+        }
+
         boolean isBot = false;
         String sessionId = null;
         String clientIp = request.getRemoteAddr();
-        String clientIdentifier = getClientIdentifier(request.getHost(), request.getContext(), clientIp);
+        String clientIdentifier = getClientIdentifier(host, request.getContext(), clientIp);
 
-        if (log.isDebugEnabled()) {
-            log.debug(request.hashCode() + ": ClientIdentifier=" + clientIdentifier + ", RequestedSessionId="
-                    + request.getRequestedSessionId());
+        if (log.isTraceEnabled()) {
+            log.trace(request.hashCode() + ": ClientIdentifier=" + clientIdentifier + ", RequestedSessionId=" +
+                    request.getRequestedSessionId());
         }
 
         // If the incoming request has a valid session ID, no action is required
@@ -199,15 +205,15 @@ public class CrawlerSessionManagerValve extends ValveBase {
             // If more than one UA header - assume not a bot
             if (uaHeader != null && !uaHeaders.hasMoreElements()) {
 
-                if (log.isDebugEnabled()) {
-                    log.debug(request.hashCode() + ": UserAgent=" + uaHeader);
+                if (log.isTraceEnabled()) {
+                    log.trace(request.hashCode() + ": UserAgent=" + uaHeader);
                 }
 
                 if (uaPattern.matcher(uaHeader).matches()) {
                     isBot = true;
 
-                    if (log.isDebugEnabled()) {
-                        log.debug(request.hashCode() + ": Bot found. UserAgent=" + uaHeader);
+                    if (log.isTraceEnabled()) {
+                        log.trace(request.hashCode() + ": Bot found. UserAgent=" + uaHeader);
                     }
                 }
             }
@@ -215,8 +221,8 @@ public class CrawlerSessionManagerValve extends ValveBase {
             if (ipPattern != null && ipPattern.matcher(clientIp).matches()) {
                 isBot = true;
 
-                if (log.isDebugEnabled()) {
-                    log.debug(request.hashCode() + ": Bot found. IP=" + clientIp);
+                if (log.isTraceEnabled()) {
+                    log.trace(request.hashCode() + ": Bot found. IP=" + clientIp);
                 }
             }
 
@@ -225,8 +231,8 @@ public class CrawlerSessionManagerValve extends ValveBase {
                 sessionId = clientIdSessionId.get(clientIdentifier);
                 if (sessionId != null) {
                     request.setRequestedSessionId(sessionId);
-                    if (log.isDebugEnabled()) {
-                        log.debug(request.hashCode() + ": SessionID=" + sessionId);
+                    if (log.isTraceEnabled()) {
+                        log.trace(request.hashCode() + ": SessionID=" + sessionId);
                     }
                 }
             }
@@ -240,20 +246,18 @@ public class CrawlerSessionManagerValve extends ValveBase {
                 HttpSession s = request.getSession(false);
                 if (s != null) {
                     clientIdSessionId.put(clientIdentifier, s.getId());
-                    sessionIdClientId.put(s.getId(), clientIdentifier);
                     // #valueUnbound() will be called on session expiration
                     s.setAttribute(this.getClass().getName(),
                             new CrawlerHttpSessionBindingListener(clientIdSessionId, clientIdentifier));
                     s.setMaxInactiveInterval(sessionInactiveInterval);
 
-                    if (log.isDebugEnabled()) {
-                        log.debug(request.hashCode() + ": New bot session. SessionID=" + s.getId());
+                    if (log.isTraceEnabled()) {
+                        log.trace(request.hashCode() + ": New bot session. SessionID=" + s.getId());
                     }
                 }
             } else {
-                if (log.isDebugEnabled()) {
-                    log.debug(
-                            request.hashCode() + ": Bot session accessed. SessionID=" + sessionId);
+                if (log.isTraceEnabled()) {
+                    log.trace(request.hashCode() + ": Bot session accessed. SessionID=" + sessionId);
                 }
             }
         }
@@ -271,17 +275,11 @@ public class CrawlerSessionManagerValve extends ValveBase {
         return result.toString();
     }
 
-    private static class CrawlerHttpSessionBindingListener implements HttpSessionBindingListener, Serializable {
+    private record CrawlerHttpSessionBindingListener(Map<String,String> clientIdSessionId, String clientIdentifier)
+            implements HttpSessionBindingListener, Serializable {
+        @Serial
         private static final long serialVersionUID = 1L;
 
-        private final transient Map<String, String> clientIdSessionId;
-        private final transient String clientIdentifier;
-
-        private CrawlerHttpSessionBindingListener(Map<String, String> clientIdSessionId, String clientIdentifier) {
-            this.clientIdSessionId = clientIdSessionId;
-            this.clientIdentifier = clientIdentifier;
-        }
-
         @Override
         public void valueUnbound(HttpSessionBindingEvent event) {
             if (clientIdentifier != null && clientIdSessionId != null) {
