@@ -20,6 +20,7 @@ import java.io.ByteArrayOutputStream;
 import java.io.CharConversionException;
 import java.io.IOException;
 import java.io.OutputStreamWriter;
+import java.io.Serial;
 import java.nio.charset.Charset;
 import java.nio.charset.StandardCharsets;
 
@@ -28,18 +29,13 @@ import org.apache.tomcat.util.res.StringManager;
 /**
  * All URL decoding happens here. This way we can reuse, review, optimize without adding complexity to the buffers. The
  * conversion will modify the original buffer.
- *
- * @author Costin Manolache
  */
 public final class UDecoder {
 
     private static final StringManager sm = StringManager.getManager(UDecoder.class);
 
-    @Deprecated
-    public static final boolean ALLOW_ENCODED_SLASH = Boolean
-            .parseBoolean(System.getProperty("org.apache.tomcat.util.buf.UDecoder.ALLOW_ENCODED_SLASH", "false"));
-
     private static class DecodeException extends CharConversionException {
+        @Serial
         private static final long serialVersionUID = 1L;
 
         DecodeException(String s) {
@@ -105,10 +101,8 @@ public final class UDecoder {
      * URLDecode, will modify the source. Assumes source bytes are encoded using a superset of US-ASCII as per RFC 7230.
      *
      * @param mb                            The URL encoded bytes
-     * @param encodedSolidusHandling        How should the %2f sequence handled by the decoder? For query strings this
-     *                                          parameter will be ignored and the %2f sequence will be decoded
-     * @param encodedReverseSolidusHandling How should the %5c sequence handled by the decoder? For query strings this
-     *                                          parameter will be ignored and the %5c sequence will be decoded
+     * @param encodedSolidusHandling        How should the %2f sequence handled by the decoder?
+     * @param encodedReverseSolidusHandling How should the %5c sequence handled by the decoder?
      *
      * @throws IOException Invalid %xx URL encoding
      */
@@ -211,201 +205,6 @@ public final class UDecoder {
 
     // -------------------- Additional methods --------------------
 
-    /**
-     * In-buffer processing - the buffer will be modified.
-     * <p>
-     * <b>WARNING:</b> This method assumes US-ASCII encoding.
-     *
-     * @param mb    The URL encoded chars
-     * @param query <code>true</code> if this is a query string
-     *
-     * @throws IOException Invalid %xx URL encoding
-     *
-     * @deprecated Unused. Will be removed in Tomcat 10
-     */
-    @Deprecated
-    public void convert(CharChunk mb, boolean query) throws IOException {
-        // log( "Converting a char chunk ");
-        int start = mb.getOffset();
-        char buff[] = mb.getBuffer();
-        int cend = mb.getEnd();
-
-        int idx = CharChunk.indexOf(buff, start, cend, '%');
-        int idx2 = -1;
-        if (query) {
-            idx2 = CharChunk.indexOf(buff, start, (idx >= 0 ? idx : cend), '+');
-        }
-        if (idx < 0 && idx2 < 0) {
-            return;
-        }
-
-        // idx will be the smallest positive index ( first % or + )
-        if ((idx2 >= 0 && idx2 < idx) || idx < 0) {
-            idx = idx2;
-        }
-
-        final boolean noSlash = !(ALLOW_ENCODED_SLASH || query);
-
-        for (int j = idx; j < cend; j++, idx++) {
-            if (buff[j] == '+' && query) {
-                buff[idx] = (' ');
-            } else if (buff[j] != '%') {
-                buff[idx] = buff[j];
-            } else {
-                // read next 2 digits
-                if (j + 2 >= cend) {
-                    // invalid
-                    throw EXCEPTION_EOF;
-                }
-                char b1 = buff[j + 1];
-                char b2 = buff[j + 2];
-                if (!isHexDigit(b1) || !isHexDigit(b2)) {
-                    throw EXCEPTION_NOT_HEX_DIGIT;
-                }
-
-                j += 2;
-                int res = x2c(b1, b2);
-                if (noSlash && (res == '/')) {
-                    throw EXCEPTION_SLASH;
-                }
-                buff[idx] = (char) res;
-            }
-        }
-        mb.setEnd(idx);
-    }
-
-    /**
-     * URLDecode, will modify the source.
-     * <p>
-     * <b>WARNING:</b> This method assumes US-ASCII encoding.
-     *
-     * @param mb    The URL encoded String, bytes or chars
-     * @param query <code>true</code> if this is a query string
-     *
-     * @throws IOException Invalid %xx URL encoding
-     *
-     * @deprecated Unused. Will be removed in Tomcat 10
-     */
-    @Deprecated
-    public void convert(MessageBytes mb, boolean query) throws IOException {
-
-        switch (mb.getType()) {
-            case MessageBytes.T_STR:
-                String strValue = mb.toString();
-                if (strValue == null) {
-                    return;
-                }
-                try {
-                    mb.setString(convert(strValue, query));
-                } catch (RuntimeException ex) {
-                    throw new DecodeException(ex.getMessage());
-                }
-                break;
-            case MessageBytes.T_CHARS:
-                CharChunk charC = mb.getCharChunk();
-                convert(charC, query);
-                break;
-            case MessageBytes.T_BYTES:
-                ByteChunk bytesC = mb.getByteChunk();
-                convert(bytesC, query);
-                break;
-        }
-    }
-
-    /**
-     * %xx decoding of a string.
-     * <p>
-     * <b>WARNING:</b> This method assumes US-ASCII encoding.
-     * <p>
-     * FIXME: this is inefficient.
-     *
-     * @param str   The URL encoded string
-     * @param query <code>true</code> if this is a query string
-     *
-     * @return the decoded string
-     *
-     * @deprecated Unused. Will be removed in Tomcat 10
-     */
-    @Deprecated
-    public String convert(String str, boolean query) {
-        if (str == null) {
-            return null;
-        }
-
-        if ((!query || str.indexOf('+') < 0) && str.indexOf('%') < 0) {
-            return str;
-        }
-
-        final boolean noSlash = !(ALLOW_ENCODED_SLASH || query);
-
-        StringBuilder dec = new StringBuilder(); // decoded string output
-        int strPos = 0;
-        int strLen = str.length();
-
-        dec.ensureCapacity(str.length());
-        while (strPos < strLen) {
-            int laPos; // lookahead position
-
-            // look ahead to next URLencoded metacharacter, if any
-            for (laPos = strPos; laPos < strLen; laPos++) {
-                char laChar = str.charAt(laPos);
-                if ((laChar == '+' && query) || (laChar == '%')) {
-                    break;
-                }
-            }
-
-            // if there were non-metacharacters, copy them all as a block
-            if (laPos > strPos) {
-                dec.append(str.substring(strPos, laPos));
-                strPos = laPos;
-            }
-
-            // shortcut out of here if we're at the end of the string
-            if (strPos >= strLen) {
-                break;
-            }
-
-            // process next metacharacter
-            char metaChar = str.charAt(strPos);
-            if (metaChar == '+') {
-                dec.append(' ');
-                strPos++;
-                continue;
-            } else if (metaChar == '%') {
-                // We throw the original exception - the super will deal with
-                // it
-                // try {
-                char res = (char) Integer.parseInt(str.substring(strPos + 1, strPos + 3), 16);
-                if (noSlash && (res == '/')) {
-                    throw new IllegalArgumentException(sm.getString("uDecoder.noSlash"));
-                }
-                dec.append(res);
-                strPos += 3;
-            }
-        }
-
-        return dec.toString();
-    }
-
-
-    /**
-     * Decode and return the specified URL-encoded String. When the byte array is converted to a string, UTF-8 is used.
-     * This may be different than some other servers. It is assumed the string is not a query string.
-     *
-     * @param str The url-encoded string
-     *
-     * @return the decoded string
-     *
-     * @exception IllegalArgumentException if a '%' character is not followed by a valid 2-digit hexadecimal number
-     *
-     * @deprecated Unused. This will be removed in Tomcat 10 onwards
-     */
-    @Deprecated
-    public static String URLDecode(String str) {
-        return URLDecode(str, StandardCharsets.UTF_8);
-    }
-
-
     /**
      * Decode and return the specified URL-encoded String. It is assumed the string is not a query string.
      *
@@ -546,7 +345,7 @@ public final class UDecoder {
             }
             osw.flush();
 
-            return baos.toString(charset.name());
+            return baos.toString(charset);
         } catch (IOException ioe) {
             throw new IllegalArgumentException(sm.getString("uDecoder.urlDecode.conversionError", str, charset.name()),
                     ioe);
