@@ -198,23 +198,6 @@ public class DiskFileItem implements FileItem {
         }
     }
 
-    /**
-     * Removes the file contents from the temporary storage.
-     *
-     * @throws Throwable Thrown by {@link Object#finalize()}.
-     */
-    @Override
-    protected void finalize() throws Throwable {
-        if (dfos == null || dfos.isInMemory()) {
-            return;
-        }
-        final File outputFile = dfos.getFile();
-        if (outputFile != null && outputFile.exists()) {
-            outputFile.delete();
-        }
-        super.finalize();
-    }
-
     /**
      * Gets the contents of the file as an array of bytes. If the contents of the file were not yet cached in memory,
      * they will be loaded from the disk storage and cached.
@@ -252,7 +235,7 @@ public class DiskFileItem implements FileItem {
         final ParameterParser parser = new ParameterParser();
         parser.setLowerCaseNames(true);
         // Parameter parser can handle null input
-        final Map<String, String> params = parser.parse(getContentType(), ';');
+        final Map<String,String> params = parser.parse(getContentType(), ';');
         return params.get("charset");
     }
 
