@@ -29,6 +29,8 @@ import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.buf.HexUtils;
 import org.apache.tomcat.util.http.parser.HttpParser;
 import org.apache.tomcat.util.net.openssl.ciphers.Cipher;
+import org.apache.tomcat.util.net.openssl.ciphers.Group;
+import org.apache.tomcat.util.net.openssl.ciphers.SignatureScheme;
 import org.apache.tomcat.util.res.StringManager;
 
 /**
@@ -45,10 +47,17 @@ public class TLSClientHelloExtractor {
     private final String sniValue;
     private final List<String> clientRequestedApplicationProtocols;
     private final List<String> clientRequestedProtocols;
+    private final List<Group> clientSupportedGroups;
+    private final List<SignatureScheme> clientSignatureSchemes;
 
     private static final int TLS_RECORD_HEADER_LEN = 5;
 
     private static final int TLS_EXTENSION_SERVER_NAME = 0;
+    private static final int TLS_EXTENSION_SUPPORTED_GROUPS = 10;
+    // Note: Signature algorithms is the name of the extension
+    // Starting with TLS 1.3, this contains signature schemes
+    // For TLS before 1.3, this contains signature algorithms
+    private static final int TLS_EXTENSION_SIGNATURE_ALGORITHMS = 13;
     private static final int TLS_EXTENSION_ALPN = 16;
     private static final int TLS_EXTENSION_SUPPORTED_VERSION = 43;
 
@@ -77,6 +86,8 @@ public class TLSClientHelloExtractor {
         List<String> clientRequestedCipherNames = new ArrayList<>();
         List<String> clientRequestedApplicationProtocols = new ArrayList<>();
         List<String> clientRequestedProtocols = new ArrayList<>();
+        List<Group> clientSupportedGroups = new ArrayList<>();
+        List<SignatureScheme> clientSignatureSchemes = new ArrayList<>();
         String sniValue = null;
         try {
             // Switch to read mode.
@@ -158,6 +169,12 @@ public class TLSClientHelloExtractor {
                         sniValue = readSniExtension(netInBuffer);
                         break;
                     }
+                    case TLS_EXTENSION_SUPPORTED_GROUPS:
+                        readSupportedGroups(netInBuffer, clientSupportedGroups);
+                        break;
+                    case TLS_EXTENSION_SIGNATURE_ALGORITHMS:
+                        readSignatureSchemes(netInBuffer, clientSignatureSchemes);
+                        break;
                     case TLS_EXTENSION_ALPN:
                         readAlpnExtension(netInBuffer, clientRequestedApplicationProtocols);
                         break;
@@ -182,6 +199,14 @@ public class TLSClientHelloExtractor {
             this.clientRequestedApplicationProtocols = clientRequestedApplicationProtocols;
             this.sniValue = sniValue;
             this.clientRequestedProtocols = clientRequestedProtocols;
+            this.clientSupportedGroups = clientSupportedGroups;
+            this.clientSignatureSchemes = clientSignatureSchemes;
+            if (log.isTraceEnabled()) {
+                log.trace("TLS Client Hello: " + clientRequestedCiphers + " Names " + clientRequestedCipherNames +
+                        " Protocols " + clientRequestedApplicationProtocols + " sniValue " + sniValue +
+                        " clientRequestedProtocols " + clientRequestedProtocols + " clientSupportedGroups " + clientSupportedGroups +
+                        " clientSignatureSchemes " + clientSignatureSchemes);
+            }
             // Whatever happens, return the buffer to its original state
             netInBuffer.limit(limit);
             netInBuffer.position(pos);
@@ -242,6 +267,24 @@ public class TLSClientHelloExtractor {
     }
 
 
+    public List<Group> getClientSupportedGroups() {
+        if (result == ExtractorResult.COMPLETE || result == ExtractorResult.NOT_PRESENT) {
+            return clientSupportedGroups;
+        } else {
+            throw new IllegalStateException(sm.getString("sniExtractor.tooEarly"));
+        }
+    }
+
+
+    public List<SignatureScheme> getClientSignatureSchemes() {
+        if (result == ExtractorResult.COMPLETE || result == ExtractorResult.NOT_PRESENT) {
+            return clientSignatureSchemes;
+        } else {
+            throw new IllegalStateException(sm.getString("sniExtractor.tooEarly"));
+        }
+    }
+
+
     private static ExtractorResult handleIncompleteRead(ByteBuffer bb) {
         if (bb.limit() == bb.capacity()) {
             // Buffer not big enough
@@ -413,6 +456,34 @@ public class TLSClientHelloExtractor {
     }
 
 
+    private static void readSupportedGroups(ByteBuffer bb, List<Group> groups) {
+        // First 2 bytes are size of the group list
+        int toRead = bb.getChar() / 2;
+        // Then the list of groups
+        for (int i = 0; i < toRead; i++) {
+            char id = bb.getChar();
+            Group group = Group.valueOf(id);
+            if (group != null) {
+                groups.add(group);
+            }
+        }
+    }
+
+
+    private static void readSignatureSchemes(ByteBuffer bb, List<SignatureScheme> signatureSchemes) {
+        // First 2 bytes are size of the signature algorithm list
+        int toRead = bb.getChar() / 2;
+        // Then the list of schemes
+        for (int i = 0; i < toRead; i++) {
+            char id = bb.getChar();
+            SignatureScheme signatureScheme = SignatureScheme.valueOf(id);
+            if (signatureScheme != null) {
+                signatureSchemes.add(signatureScheme);
+            }
+        }
+    }
+
+
     public enum ExtractorResult {
         COMPLETE,
         NOT_PRESENT,
