@@ -20,8 +20,6 @@ import java.lang.ref.WeakReference;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationHandler;
 import java.lang.reflect.Proxy;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
 import java.sql.Connection;
 import java.sql.SQLException;
 import java.util.Collections;
@@ -115,7 +113,7 @@ public class ConnectionPool {
     /**
      * Executor service used to cancel Futures
      */
-    private ThreadPoolExecutor cancellator = new ThreadPoolExecutor(0,1,1000,TimeUnit.MILLISECONDS,new LinkedBlockingQueue<Runnable>());
+    private ThreadPoolExecutor cancellator = new ThreadPoolExecutor(0,1,1000,TimeUnit.MILLISECONDS,new LinkedBlockingQueue<>());
 
     /**
      * reference to the JMX mbean
@@ -1543,8 +1541,9 @@ public class ConnectionPool {
                 // Create the timer thread in a PrivilegedAction so that a
                 // reference to the web application class loader is not created
                 // via Thread.inheritedAccessControlContext
-                PrivilegedAction<Timer> pa = new PrivilegedNewTimer();
-                poolCleanTimer = AccessController.doPrivileged(pa);
+                poolCleanTimer = new Timer("Tomcat JDBC Pool Cleaner[" +
+                        System.identityHashCode(ConnectionPool.class.getClassLoader()) + ":"+
+                        System.currentTimeMillis() + "]", true);
             } finally {
                 Thread.currentThread().setContextClassLoader(loader);
             }
@@ -1566,14 +1565,6 @@ public class ConnectionPool {
         }
     }
 
-    private static class PrivilegedNewTimer implements PrivilegedAction<Timer> {
-        @Override
-        public Timer run() {
-            return new Timer("Tomcat JDBC Pool Cleaner["+ System.identityHashCode(ConnectionPool.class.getClassLoader()) + ":"+
-                    System.currentTimeMillis() + "]", true);
-        }
-    }
-
     // Testing use only
     public static synchronized Set<TimerTask> getPoolCleaners() {
         return Collections.<TimerTask>unmodifiableSet(cleaners);
